;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================

ORG 4084

@UDGNum			;UDG number to print
	00

@Jumps
	JR ConvertBMP	;16517, Convert 64x64 pixel BMP to font format
	JR SwapBW	;16519, Swap Black and White pixels
	JR Transfer	;16521, Load BANK1 then turn on UDGs
	JR UDG4x4	;16523, Print 4x4 character UDG
	JR UDG2x2	;16525, Print 2x2 character UDG

;================================
;Convert BMP data to BANK2 font
;================================
@ConvertBMP
	LD HL,803E	;BMP data
	LD DE,2200	;Destination is BANK2
@Loop0
	LD B,08		;8x8 characters
@Loop1
	PUSH BC		;Push counter
	PUSH HL		;PUSH BMP data address
	LD B,08		;Counter, 8 bytes/character
@Loop2
	LD A,(HL)	;BMP data
	LD (DE),A	;Move to font
	INC DE		;Next font address
	LD A,D		;512 bytes to process
	CP 24		;Done?
	JR NZ MoreBMP
	POP HL		;Clear stack
	POP BC
	LD BC,0000	;Return 0 to BASIC
	RET
@MoreBMP
	LD A,B		;Save counter
	LD BC,0008
	ADD HL,BC
	LD B,A		;Restore counter
	DJNZ Loop2
	POP HL
	INC HL
	POP BC
	DJNZ Loop1
	LD BC,0038	;Next row offset, 56 bytes
	ADD HL,BC	;Start of next row
	JR Loop0
	RET
;================================
;Swap B/W
;================================
@SwapBW
	LD HL,2200	;Font Bank2 address
@Loop3
	LD A,(HL)
	CPL
	LD (HL),A
	INC HL
	LD A,H
	CP 24
	RET Z
	JR Loop3
;================================
;Turn on UDGs
;================================
@UDGOn
	LD A,21		;CHR128 mode
@SetI
	LD I,A
	RET
;================================
;Turn off UDGs
;================================
@UDGOff
	LD A,1E
	JR SetI
;================================
;Transfer font data to BANK1
;================================
@Transfer
	LD HL,+2	;Locate start of font data, LINE 2
	CALL 09D8	;Get Line 2 start address
	INC HL		;Skip header
	INC HL
	INC HL
	INC HL
	INC HL
	INC HL
	INC HL
	LD DE,2000	;Bank1 address
	LD BC,0200	;512 bytes to move
	LDIR
	JR UDGOn	;Turn on UDGs
;================================
;4x4 UDG print
;================================
@UDG4x4
	LD A,(UDGNum)	;Get UDG number
	DEC A
	LD HL,0000	;HL=0
	JR Z UDGAddr
	LD B,A		;B=UDG Number 1,2 or 3
	LD C,10		;Calculation UDG location
	XOR A		;Clear CY
@UDGFind
	ADD A,C		;Find UDG address, A=16*UDG number
	DJNZ UDGFind
	LD L,A
@UDGAddr
	LD DE,UDG1	;UDG table base address
	ADD HL,DE
	EX DE,HL	;DE=UDG defintion
	LD HL,(400E)	;HL=Print position
	LD B,04		;Rows done counter
@PrintRow
	PUSH BC		;Push row counter
	LD B,04		;Chrs printed counter
@PrintChr
	BIT 6,(HL)	;N/L?
	JR Z NotNL
	INC HL		;Skip N/L character
@NotNL
	LD A,(DE)	;A=UDG chr to print
	LD (HL),A	;Print character
	INC HL		;Next print position
	INC DE		;Next chr to print
	DJNZ PrintChr
	LD C,1D		;Next line offset
	ADD HL,BC	;Next line address
	POP BC		;POP line counter
	DJNZ PrintRow
	LD BC,0000	;Return 0 to BASIC
	RET
;+---+---+---+---+
;|NW     |     NE|
;|       |       |
;|       |       |
;+---+---+---+---+ 
;|       |       |
;|       |       |
;|SW     |     SE|
;+---+---+---+---+ 

@UDG4BY4
	80 81 82 83	;NW
	88 89 8A 8B
	90 91 92 93
	98 99 9A 9B
	84 85 86 87	;NE
	8C 8D 8E 8F
	94 95 96 97
	9C 9D 9E 9F
	A0 A1 A2 A3	;SW
	A8 A9 AA AB
	B0 B1 B2 B3
	B8 B9 BA BB
	A4 A5 A6 A7	;SE
	AC AD AE AF
	B4 B5 B6 B7
	BC BD BE BF

;+---+---+---+---+
;+ 1 | 2 | 3 | 4 | Line 1
;+---+---+---+---+ 
;+ 5 | 6 | 7 | 8 | Line 2
;+---+---+---+---+ 
;+ 9 | A | B | C | Line 3
;+---+---+---+---+ 
;+ D | E | F | G | Line 4
;+---+---+---+---+ 

@UDG2by2
	80 81 88 89	;1
	82 83 8A 8B	;2
	84 84 8C 8D	;3
	86 87 8E 8F	;4

	90 91 98 99	;5
	92 93 9A 9B	;6
	94 94 9C 9D	;7
	96 97 9E 9F	;8

	A0 A1 A8 A9	;9
	A2 A3 AA AB	;A
	A4 A5 AC AD	;B
	A6 A7 AE 8F	;C

	B0 B1 B8 B9	;D
	B2 B3 BA BB	;E
	B4 B5 BC BD	;F
	B6 B7 BE BF	;G
	
	
